#ifndef PACMAN_H
#define PACMAN_H

#include <iostream>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>
#include "cmdline_options.h"
#include "constants.h"
#include "game.h"
#include "ghost_figur.h"
#include "pacman_figur.h"
#include "menu_main.h"
#include "highscore.h"

#endif

